/*
THIS PROGRAM WORKS WITH PulseSensorAmped_Arduino-xx ARDUINO CODE
THE PULSE DATA WINDOW IS SCALEABLE WITH SCROLLBAR AT BOTTOM OF SCREEN
PRESS 'S' OR 's' KEY TO SAVE A PICTURE OF THE SCREEN IN SKETCH FOLDER (.jpg)
MADE BY JOEL MURPHY AUGUST, 2012
UPDATED BY JOEL MURPHY JULY 2016 WITH SERIAL PORT LOCATOR TOOL

THIS CODE PROVIDED AS IS, WITH NO CLAIMS OF FUNCTIONALITY OR EVEN IF IT WILL WORK
      WYSIWYG
      
Modified by Sahan Wickramaarachchi to visualize data coming from light sensor.
*/


import processing.serial.*;

//For serial communication and transformation between counter.
Serial myPort;
private static final String PORT = "COM8";
private static final int FREQ_MHZ = 20; //Frequency in MHz of the counter
private static final int SCALER = 150; //Counter scaler
private static final float EPS =0.1;


//Font and shapes and colors
PFont font;
color eggshell = color(255, 255, 255);

//THE SIZE OF THE DATA WINDOWS
int PulseWindowWidth = 490;
int PulseWindowHeight = 512;
int VERTICAL_OFFSET= 50; //Number of pixels from the botomm of the window which is considered as origin of distance.

//Data structures
int[] RawY;      // HOLDS HEARTBEAT WAVEFORM DATA BEFORE SCALING
int distance;    // The last distance visualized.

private final int MAX_DISTANCE=300;


//Local variables for the serial events
int byte1, byte2, byte3, byte4, low, high;
int filt; 
void serialEvent(Serial p){
    byte1 = byte2;
    byte2 = byte3;
    byte3 = byte4;
    byte4 = low;
    low = high;
    high = p.read();        
    
    if ((byte1 == 1) & (byte2 == 2) & (byte3 == 3) & (byte4 ==4)){
      //Reset the preamble.
      byte1=byte2=byte3=byte4=0;
      int counter = ((256*high)+low);
      println("Counter is "+ counter);
      //Filter
      filt = int((1-EPS)*filt+EPS*counter);
      int us = counter*SCALER/FREQ_MHZ;//microseconds
      int cm = int(us/58);
      //Do not visualize more than 100cm
      if (cm > MAX_DISTANCE){
        cm = MAX_DISTANCE;
      }
      updateGraph(cm);
      println("New value: "+ cm);
      
    }

}

void updateGraph(int value){
 distance = value;
 //Add the point to the graph
 //Shift all to the left (new value will be added at the end), instead of using for loop I utilize the arraycopy method that it is much faster.
 int[] buffer = new int[PulseWindowWidth];
 arrayCopy(RawY, 1, buffer, 0, RawY.length-1);
 //Make a map: distance values [0,200], vertical values [0,pulsewindowheight-verticaloffset]
 value = int(map(value,0,MAX_DISTANCE, 0, PulseWindowHeight-VERTICAL_OFFSET));
 buffer[buffer.length-1]=value;
 RawY=buffer;
 println("Added: " +RawY[RawY.length-1] );
 
}

void setup() {
   //Serial communication setup
  //Setup serial communication
  String portName = Serial.list()[0];
  println("Port Name "+ portName); 
  myPort = new Serial(this, PORT, 9600);
  
  //Setup the UI
  size(700, 600);  // Stage size
  frameRate(50); //The PCB sends one frame (6 bytes/500ms)-> This frame rate should be enough. 
  //Text control and alignment
  font = loadFont("Arial-BoldMT-24.vlw");
  textFont(font);
  textAlign(CENTER);
  rectMode(CENTER);
  ellipseMode(CENTER);

  //Initialize the array of data with 0
  RawY = new int[PulseWindowWidth];          // initialize raw pulse waveform array
  for (int i=0; i<RawY.length; i++){
    RawY[i] = 0 ; // initialize the pulse values to 0;
 }

 background(0);
 noStroke();
 // DRAW OUT THE PULSE WINDOW AND BPM WINDOW RECTANGLES
 drawDataWindows();

}

void drawDataWindows(){
    // DRAW OUT THE PULSE WINDOW AND BPM WINDOW RECTANGLES
    fill(eggshell);  // color for the window background
    rect(255,height/2,PulseWindowWidth,PulseWindowHeight);
    
}

void draw() {

  background(0);
  noStroke();
  drawDataWindows();
  drawPulseWaveform();

// PRINT THE DATA AND VARIABLE VALUES
  fill(eggshell);                                       // get ready to print text
  text("LIGHT SENSOR",245,30);     // tell them what you are
  text( "value="+distance ,600,200);                           // print the distance


}  //end of draw loop




void drawPulseWaveform(){
  // DRAW THE PULSE WAVEFORM
  // prepare pulse data points
  
  stroke(250,0,0);                               // red is a good color for the pulse waveform
  noFill();
  beginShape();                                  // using beginShape() renders fast
  for (int x = 1; x < RawY.length-1; x++) {
    vertex(x+10, height-VERTICAL_OFFSET-RawY[x]);//draw a line connecting the data points
  }
  endShape();
}
